<?php declare(strict_types=1);
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */
namespace Amazonfulfillment\AmazonMCF\Block\Adminhtml;

use Magento\Backend\Block\Template;
use Magento\Backend\Block\Template\Context;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Store\Api\WebsiteRepositoryInterface;
use Magento\Directory\Api\CountryInformationAcquirerInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use Amazonfulfillment\AmazonMCF\Helper\Data;
use Amazonfulfillment\AmazonMCF\Model\ConfigManager;
use Magento\Framework\App\Request\Http;
use Magento\InventoryApi\Api\Data\SourceInterface;
use Magento\InventoryApi\Api\SourceRepositoryInterface;

class Index extends Template
{
    private const SELLER_CENTRAL = "https://sellercentral.amazon.com/";
    private const SELLER_SIGNUP = "https://supplychain.amazon.com/sign-up";
    private const CONTACT_US = "https://supplychain.amazon.com/contact-us";
    private const SELLER_HELP = "https://sellercentral.amazon.com/gp/help/help.html?itemID=G200333160&language=en_US";
    private const SHIPPING_HELP = "https://sellercentral.amazon.com/gp/help/G891";
    private const CRON_DOC = "https://docs.magento.com/m2/ee/user_guide/system/cron.html";
    private const CREATE_ATTR = "https://docs.magento.com/m2/ee/user_guide/sales-channels/asc/creating-attributes.html";
    private const CURR_SETUP = "https://docs.magento.com/m2/ce/user_guide/configuration/general/currency-setup.html";
    private const CURR_UPDATE = "https://docs.magento.com/m2/ce/user_guide/stores/currency-update.html";
    /**
     * @var UrlInterface
     */
    private $urlBuilder;
    /**
     * @var StoreManagerInterface
     */
    private $storeManagerInterface;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var CountryInformationAcquirerInterface
     */
    private $countryInformationInterface;
    /**
     * @var Data
     */
    private $helper;
    /**
     * @var ConfigManager
     */
    private $configManager;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var SourceRepositoryInterface
     */
    private $sourceRepository;
    /**
     * @param Context $context
     * @param ScopeConfigInterface $scopeConfig
     * @param CountryInformationAcquirerInterface $countryInformationInterface
     * @param Data $helper
     * @param ConfigManager $configManager
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param SourceRepositoryInterface $sourceRepository
     * @param Http $request
     */
    public function __construct(
        Context $context,
        ScopeConfigInterface $scopeConfig,
        CountryInformationAcquirerInterface $countryInformationInterface,
        Data $helper,
        ConfigManager $configManager,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        SourceRepositoryInterface $sourceRepository,
        Http $request
    ) {
        $this->urlBuilder = $context->getUrlBuilder();
        $this->scopeConfig = $scopeConfig;
        $this->countryInformationInterface = $countryInformationInterface;
        $this->helper = $helper;
        $this->configManager = $configManager;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->sourceRepository = $sourceRepository;
        $this->request = $request;
        parent::__construct($context);
    }

    /**
     * Generate url by route and parameters
     *
     * @param   string $route
     * @param   array $params
     * @return  string
     */
    public function getUrl($route = '', $params = [])
    {
        return $this->urlBuilder->getUrl($route, $params);
    }

    /**
     * Save Url
     */
    public function getSaveUrl()
    {
        return $this->getUrl('amazonmcf/setup/save');
    }

    /**
     * MCF allowed countries
     */
    public function getAmcfCountries()
    {
        return $this->helper->getAmcfCountries();
    }

    /**
     * Versions
     */
    public function getAmcfVersions()
    {
        return $this->helper->getAmcfVersions();
    }
    /**
     * MCF connect URL
     *
     * @param int $website
     */
    public function getConnectUrl($website)
    {
        $countryName = null;
        $countryCode = $this->helper->getConfig('country', $website);
        $applicationId = $this->helper->getApplicationId();
        $url = $this->helper->getMidUrl().'connect?';
        $url .= "application_id=".$applicationId;
        $url .= "&endpoint=".$countryCode;
        $url .= "&website=".$website;
        $url .= "&domain=".$this->getBaseUrl();
        return $url;
    }
    /**
     * Store Delete URL
     *
     * @param string $websiteId
     */
    public function getDeleteUrl($websiteId = null)
    {
        $route = "amazonmcf/setup/delete";
        return $this->urlBuilder->getUrl($route, ['website' => $websiteId]);
    }
    /**
     * Generate url by route and parameters for re auth URL to set AMCF App
     *
     * @param   string $route
     * @param   array $params
     * @return  string
     */
    public function getRetrunUrl()
    {
        $route = "amazonmcf/setup/reauthconfirm";
        return $this->urlBuilder->getBaseUrl().$route;
    }

    /**
     * Disconnect feature redirect
     *
     * @param string $websiteId
     */
    public function getDisconnectUrl($websiteId = null)
    {
        $route = "amazonmcf/setup/disconnect";
        return $this->urlBuilder->getUrl($route, ['website' => $websiteId]);
    }

    /**
     * Is connected with MCF
     */
    public function isConnected()
    {
        $connected = $this->helper->isConnected();
        if ($connected) {
            return $connected;
        }
        return false;
    }
    /**
     * Is connected with MCF
     *
     * @param int $websiteId
     * @param string $path
     * @return boolean
     */
    public function isConnectedExist($websiteId, $path)
    {
        $connected = $this->helper->isConnectedExist($websiteId, $path);
        if ($connected) {
            return $connected;
        }
        return false;
    }
    /**
     * Check current Page
     *
     * @param   string $current
     * @return  string
     */
    public function checkCurrentPage($current)
    {
        if ($this->request->getRouteName() == 'amazonmcf' && $this->request->getControllerName() == $current) {
            return 'active';
        }
        return '';
    }

     /**
      * Return array of active websites
      */
    public function getWebsites()
    {
        return $this->helper->getWebsites();
    }

    /**
     * Return array of active connections
     */
    public function getActiveConnections()
    {
        return $this->helper->getActiveConnections();
    }
    /**
     * Return show add store button status
     */
    public function showAddStore()
    {
        if (count($this->getActiveConnections()) < count($this->getWebsites())) {
            return true;
        }
        return false;
    }

    /**
     * Get Website URL
     *
     * @param int $websiteId
     * @return string
     */
    public function getWebsiteUrl($websiteId)
    {
        $url = $this->helper->getConfig('base_url', $websiteId);
        if (!empty($url)) {
            return $url;
        }
        return '';
    }
    /**
     * Get Document URLs
     *
     * @param   string $key
     * @return  string
     */
    public function getDocumentUrl($key)
    {
        if ($key == 'seller_central') {
            return self::SELLER_CENTRAL;
        }
        if ($key == 'signup') {
            return self::SELLER_SIGNUP;
        }
        if ($key == 'contact-us') {
            return self::CONTACT_US;
        }
        if ($key == 'help') {
            return self::SELLER_HELP;
        }
        if ($key == 'shipping_help') {
            return self::SHIPPING_HELP;
        }
        if ($key == 'cron_doc') {
            return self::CRON_DOC;
        }
        if ($key == 'creating_attr') {
            return self::CREATE_ATTR;
        }
        if ($key == 'currency_setup') {
            return self::CURR_SETUP;
        }
        if ($key == 'currency_update') {
            return self::CURR_UPDATE;
        }
    }
  /**
   * Get Soruce Lists
   *
   * @return  array
   */
    public function getSources()
    {
        $arrSources = [];
        $searchCriteria = $this->searchCriteriaBuilder->create();
        try {
            $sourceData = $this->sourceRepository->getList($searchCriteria);
            if ($sourceData->getTotalCount()) {
                foreach ($sourceData->getItems() as $source) {
                    $arrSources[$source->getSourceCode()] = $source->getName();
                }
                return $arrSources;
            }
        } catch (Exception $exception) {
            $this->logger->error($exception->getMessage());
        }
        return null;
    }
}
