<?php declare(strict_types = 1);
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */

namespace Amazonfulfillment\AmazonMCF\Model\Queue;

use Amazonfulfillment\AmazonMCF\Model\ConfigManager;
use Amazonfulfillment\AmazonMCF\Model\McfLogManager;
use Magento\Framework\Serialize\Serializer\Json;
use Amazonfulfillment\AmazonMCF\Logger\Logger;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Store\Model\ScopeInterface;

class ConsumerInventory
{
    /**
     * @var inventoryCount
     */
    public $inventoryCount = 0;
    
    /**
     * @var ConfigManager
     */
    private $configManager;
   
    /**
     * @var McfLogManager
     */
    private $mcfLogManager;
    /**
     * @var Logger
     */
    private $mcfLogger;
    /**
     * @var Json
     */
    private $json;
    /**
     * @var TimezoneInterface
     */
    private $date;

   /**
    * Construct function
    *
    * @param ConfigManager $configManager
    * @param McfLogManager $mcfLogManager
    * @param Logger $mcfLogger
    * @param Json $json
    * @param TimezoneInterface $date
    */
    public function __construct(
        ConfigManager $configManager,
        McfLogManager $mcfLogManager,
        Logger $mcfLogger,
        Json $json,
        TimezoneInterface $date
    ) {
        $this->configManager = $configManager;
        $this->mcfLogManager = $mcfLogManager;
        $this->mcfLogger = $mcfLogger;
        $this->json = $json;
        $this->date = $date;
    }

    /**
     * Process function
     *
     * @param string $data
     */
    public function process($data)
    {
        $this->inventoryCount = 0;
        $logData = [
            'user' => 'Cron Job',
            'area' => __('Inventory Sync'),
            'type' => __('Info'),
            'details' => 'Inventory Sync queue running',
        ];
        if (isset($logData)) {
            $this->mcfLogManager->addLog($logData);
        }
        try {
            $details = $this->json->unserialize($data);
            $this->mcfLogger->notice('-INVENTORY SYNC QUEUE PROCESS START-');
            if (isset($details['nextToken'])) {
                $dataToken['nextToken'] = $details['nextToken'];
            } else {
                $dataToken['nextToken'] = '';
            }
            $result = $this->syncInventory($details['mcfItems'], $dataToken, $details['website']);
            $this->mcfLogger->notice('-INVENTORY SYNC QUEUE PROCESS END-');
            $logData = [
                'user' => 'Cron Job',
                'area' => __('Inventory Sync'),
                'type' => __('Info'),
                'details' => 'Inventory Sync queue completed.
                 <strong>'.$this->inventoryCount.'</strong> SKU(s) mapped/updated. 
                See detailed report in <strong>/var/log/amazonfulfillment_queue.log</strong>',
            ];
        } catch (\Exception $e) {
            $logData = [
                'user' => 'Cron Job',
                'area' => __('Inventory Sync'),
                'type' => __('Info'),
                'details' => 'Auto Inventory Sync queue error, See detailed  See detailed report in 
                <strong>/var/log/amazonfulfillment_queue.log</strong>',
            ];
        }
        if (isset($logData)) {
            $this->mcfLogManager->addLog($logData);
        }
    }
    /**
     * Sync Inventory function
     *
     * @param array $mcfItems
     * @param array $data
     * @param int $activeWebsite
     * @return void
     */
    public function syncInventory($mcfItems, $data, $activeWebsite)
    {
        if (isset($mcfItems['payload']['inventorySummaries'])) {
            $result = $this->updateInventory($mcfItems, $activeWebsite);
        }
        
        $mcfItems = $this->configManager->getMcfRequest('inventory', $data, $activeWebsite);
        $this->mcfLogger->notice('-Request to Fetch Seller SKUs from Amazon is Sent-');
        if (isset($mcfItems['pagination']['nextToken'])) {
            $data['nextToken'] = $mcfItems['pagination']['nextToken'];
            $this->syncInventory($mcfItems, $data, $activeWebsite);
        } else {
            $result = $this->updateInventory($mcfItems, $activeWebsite);
        }
    }
    /**
     * Update Inventory SKUS function
     *
     * @param array $mcfItems
     * @param int $activeWebsite
     * @return void
     */
    public function updateInventory($mcfItems, $activeWebsite)
    {
        $i = 0;
        try {
            if ($mcfItems != 404) {
                foreach ($mcfItems['payload']['inventorySummaries'] as $data) {
                    $stock = '';
                    $sellerSku = '';
                    $sellerSku = $data['sellerSku'];
                    $stock = $data['inventoryDetails']['fulfillableQuantity'];
                    $result = $this->configManager->updateInventory($activeWebsite, $sellerSku, $stock);
                    if ($result) {
                        $i++;
                        $this->mcfLogger->notice('Inventory of Seller SKU '. $sellerSku.' has been updated.');
                    }
                }
                $this->configManager->syncInfoUpdate('inventory', $i);
                $this->configManager->configUpdate(
                    'amazonfulfillment/inventory_config/last_synced',
                    $this->date->date()->format('Y-m-d H:i:s'),
                    ScopeInterface::SCOPE_WEBSITES,
                    $activeWebsite
                );
            } else {
                $this->mcfLogger->error("Failed to sync inventory as no matching data found in MCF account");
            }
        } catch (\Exception $e) {
            $this->mcfLogger->error("Exception Error".$e->getMessage());
        }
        $this->inventoryCount = $this->inventoryCount + $i;
    }
}
